# Temporary Storage Killer (TSK)

A Drupal module for efficiently managing and cleaning up temporary storage
data that can accumulate over time and impact site performance.

## Contents of This File

 * [Introduction](#introduction)
 * [Features](#features)
 * [Requirements](#requirements)
 * [Installation](#installation)
 * [Configuration](#configuration)
 * [Usage](#usage)
   * [Using Drush Commands](#using-drush-commands)
   * [Using the Web Interface](#using-the-web-interface)
 * [Troubleshooting](#troubleshooting)
 * [Performance Considerations](#performance-considerations)
 * [Security](#security)
 * [FAQ](#faq)
 * [Contributing](#contributing)
 * [Maintainers](#maintainers)

## Introduction

The **Temporary Storage Killer (TSK)** module provides tools to clean up
temporary storage data in Drupal.

This module consists of two components:

- **tsk**: Core module providing Drush commands for targeted temporary
  storage cleanup
- **tsk_admin**: Administrative interface for configuring and managing
  temporary storage cleanup operations

### Why Use TSK?

- **Solving issues**: Fix issues caused by unsaved changes or broken
  relationships in a layout or a view. Resolve problems when a layout or a view
  becomes inaccessible due to configuration errors. See e.g.:
    - [layout_builder module Update fix_tempstore_keys Failed](https://stackoverflow.com/q/56086691/3886962)
    - [View Tempstore Clear](https://www.drupal.org/project/view_tempstore_clear)
- **Database cleanup**: Remove stale temporary data that's no longer needed
- **Maintenance**: Automate cleanup tasks through configuration and Drush
  commands
- **Monitoring**: View and analyze temporary storage usage through the admin
  interface

## Features

### Core TSK Module
- Drush commands for precise temporary storage management
- Support for both private and shared temporary storage types
- Ability to target specific collections and keys
- Bulk deletion capabilities

### TSK Admin Module
- Web-based interface for temporary storage management
- Visual overview of all temporary storage collections
- Point-and-click deletion of storage items
- Configuration entities for automated cleanup
- Detailed storage information and statistics

## Requirements

- **Drupal**: 9.2 or higher
- **PHP**: 7.4 or higher
- **Drush**: 10.x or higher (for command-line operations)

### Optional Dependencies
- **Administration permissions**: For accessing the web interface

## Installation

Install the TSK module like any other Drupal contributed module:

### Via Composer (Recommended)
```bash
composer require drupal/tsk
drush en tsk
# Optionally enable the admin interface
drush en tsk_admin
```

### Manual Installation
1. Download the module from
   [drupal.org/project/tsk](https://www.drupal.org/project/tsk)
2. Extract to `modules/contrib/tsk`
3. Enable the module:
   ```bash
   drush en tsk
   # Optionally enable the admin interface
   drush en tsk_admin
   ```

### Via Drupal Admin Interface
1. Go to **Extend** (`/admin/modules`)
2. Click **Install new module**
3. Upload the module package or install via URL
4. Enable **Temporary Storage Killer** and optionally **TSK Admin**

## Configuration

### TSK Core Module
The core TSK module requires no configuration and works immediately after
installation.

### TSK Admin Module
The TSK Admin module provides a web interface for managing temporary storage:

1. **Access the interface**: Navigate to **Admin** → **Reports** →
   **Temporary storage collections** (`/admin/reports/tsk`)

2. **Create cleanup configurations**: Go to **Admin** → **Configuration** →
   **Development** → **Temporary Storage Killer**
   (`/admin/config/development/tsk`)

3. **Configure cleanup entities**: Click **Add TSK entity** to create a new
   cleanup configuration with the following options:
   - **Machine name**: Unique identifier for this configuration
   - **Tempstore type**: Select "private" or "shared"
   - **Collection**: Name of the temporary storage collection
     (e.g., "layout_builder.section_storage.overrides")
   - **Kill all**: Check to delete all items in the collection, or leave
     unchecked to target specific keys
   - **Key**: Specific key to delete (only used when "Kill all" is unchecked)

## Usage

### Using Drush Commands

#### Basic Temporary Storage Cleanup
```bash
# Delete all private temporary storage in the "email_tfa" collection
drush temp-store-killer private email_tfa

# Short alias for the same command
drush tsk private email_tfa

# Delete all shared temporary storage in a collection
drush tsk shared layout_builder.section_storage.overrides

# Delete a specific item by key
drush tsk shared layout_builder.section_storage.overrides my_page.11.full.en
```

#### Bulk Operations (TSK Admin)
```bash
# Execute all configured cleanup operations
drush temp-store-killer:all

# Short alias for the same command
drush tska
```

### Using the Web Interface

#### Viewing Temporary Storage
1. Navigate to **Admin** → **Reports** → **Temporary storage collections**
   (`/admin/reports/tsk`)
2. View all active temporary storage collections, their types, and item counts
3. See sample keys and expiration information
4. Use the detailed view to examine individual items

#### Manual Cleanup
1. From the temporary storage list, click **Kill all [type]** next to any
   collection
2. Or click on individual items in the detailed view and select **Kill**
3. Confirm the deletion when prompted

#### Automated Cleanup
1. Go to **Admin** → **Configuration** → **Development** →
   **Temporary Storage Killer**
2. Create cleanup configurations for regular maintenance
3. Use **Kill all** button to execute all configured cleanups
4. Alternatively, use the Drush command `drush tska` for automation

## Troubleshooting

### Common Issues

#### "Collection not found" errors
- **Cause**: The specified collection doesn't exist or has no data
- **Solution**: Use the admin interface (`/admin/reports/tsk`) to view
  available collections

#### "Database error" messages
- **Cause**: Missing key_value_expire table or database permissions
- **Solution**: Ensure your Drupal installation is complete and database
  permissions are correct

#### Permission denied
- **Cause**: User lacks the "administer tsk" permission
- **Solution**: Grant the "administer tsk" permission to appropriate user roles

#### Drush command not found
- **Cause**: Module not enabled or Drush not recognizing commands
- **Solution**:
  ```bash
  drush en tsk
  drush cache:rebuild
  ```

### Performance Considerations

#### Large Collections
- **Issue**: Deleting very large collections may timeout
- **Solution**: Use specific key targeting


### Debugging

Enable verbose output for Drush commands:
```bash
drush tsk private my_collection -v
```

Check the Drupal logs for detailed error information:
```bash
drush watchdog:show --type=tsk
```

## Performance Considerations

### When to Use TSK

- **High temporary storage usage**: Monitor `/admin/reports/tsk` for large
  collections
- **Database growth**: Large temporary storage can consume significant space
- **Memory issues**: Large collections can cause memory problems

### Best Practices

1. **Regular cleanup**: Schedule regular cleanup using cron jobs or during
   development cycles
2. **Monitor usage**: Regularly check the admin interface for growing
   collections
3. **Target specific data**: Use key-specific deletion when possible
4. **Test first**: Always test cleanup operations on non-production environments
5. **Backup consideration**: While temporary storage is ephemeral, consider
   backup implications

### Automation

Set up automated cleanup with cron:
```bash
# Add to crontab for daily cleanup
0 2 * * * /path/to/drush tska
```

## Security

### Permissions

The TSK module defines one permission:
- **"administer tsk"**: Required for all TSK operations in web interface

### Data Safety

- **Temporary data only**: TSK only affects temporary storage, not permanent
  content
- **No data recovery**: Deleted temporary storage cannot be recovered
- **User data**: Some temporary storage may contain user session data

### Recommendations

1. **Limit access**: Only grant "administer tsk" to trusted administrators
2. **Test carefully**: Always test on non-production environments first
3. **Monitor operations**: Log and monitor TSK operations
4. **Understand impact**: Know what temporary storage contains before deletion

## FAQ

### General Questions

**Q: Will TSK affect my site's content?**
A: No, TSK only affects temporary storage, not permanent content like nodes,
users, or configuration.

**Q: Can I recover deleted temporary storage?**
A: No, temporary storage deletion is permanent and cannot be undone.

**Q: How often should I run cleanup?**
A: This depends on your site's usage. Monitor the admin interface and run
cleanup as needed, typically daily or weekly.

### Technical Questions

**Q: What's the difference between private and shared temporary storage?**
A: Private temporary storage is user-specific, while shared temporary storage
is accessible across sessions.

**Q: Can I schedule automatic cleanup?**
A: Yes, use cron jobs with the `drush tska` command.

**Q: Is there a way to preview what will be deleted?**
A: Yes, use the admin interface at `/admin/reports/tsk` to see all temporary
storage before deletion.

## Contributing

We welcome contributions to improve the TSK module!

### Reporting Issues
- Submit bug reports and feature requests on the [issue queue](https://www.drupal.org/project/issues/tsk)
- Include detailed steps to reproduce any issues
- Specify Drupal and PHP versions

### Development
- Follow [Drupal coding standards](https://www.drupal.org/docs/develop/standards)
- Write tests for new functionality
- Update documentation for changes

### Testing
The module includes comprehensive test coverage:
- Functional tests for web interface
- Kernel tests for services
- Integration tests for Drush commands

### Links
- **Project page**: [drupal.org/project/tsk](https://www.drupal.org/project/tsk)
- **Issue queue**: [drupal.org/project/issues/tsk](https://www.drupal.org/project/issues/tsk)
- **Git repository**: [git.drupalcode.org/project/tsk](https://git.drupalcode.org/project/tsk)

## Maintainers

### Current Maintainers
- **[Antonín Slejška](https://www.drupal.org/u/anton%C3%ADn-slej%C5%A1ka)** - Project maintainer and original author
