<?php

namespace Drupal\tsk_admin\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Url;
use Drupal\tsk_admin\Entity\TskEntity;
use Drupal\tsk_admin\Services\TskAdminService;
use Drupal\tsk\Exception\InvalidTempstoreTypeException;
use Drupal\tsk\Exception\TempstoreKeyNotFoundException;
use Drupal\tsk\Exception\TskException;
use Drupal\tsk\Services\TskService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines TskController class.
 */
class TskAdminController extends ControllerBase {

  /**
   * Defines tskAdminService.
   *
   * @var \Drupal\tsk_admin\Services\TskAdminService
   */
  protected $tskAdminService;

  /**
   * Defines tskService.
   *
   * @var \Drupal\tsk\Services\TskService
   */
  protected $tskService;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * TskController constructor.
   *
   * @param \Drupal\tsk_admin\Services\TskAdminService $tsk_admin_service
   *   Temporary Storage Killer admin service.
   * @param \Drupal\tsk\Services\TskService $tsk_service
   *   Temporary Storage Killer service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   */
  public function __construct(TskAdminService $tsk_admin_service, TskService $tsk_service, Connection $database, DateFormatterInterface $date_formatter) {
    $this->tskAdminService = $tsk_admin_service;
    $this->tskService = $tsk_service;
    $this->database = $database;
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Drupal service container.
   *
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tsk_admin.service'),
      $container->get('tsk.service'),
      $container->get('database'),
      $container->get('date.formatter')
    );
  }

  /**
   * Delete temporary storage for the specified TSK entity.
   *
   * @param \Drupal\tsk_admin\Entity\TskEntity $tsk_entity
   *   Temporary storage entity.
   */
  public function kill(TskEntity $tsk_entity) {
    try {
      $this->tskService->kill($tsk_entity->collection, $tsk_entity->type, $tsk_entity->kill_all, $tsk_entity->key);

      if ($tsk_entity->kill_all) {
        $this->messenger()->addStatus($this->t('All @type temporary storage items in collection "@collection" have been deleted.', [
          '@type' => $tsk_entity->type,
          '@collection' => $tsk_entity->collection,
        ]));
      }
      else {
        $this->messenger()->addStatus($this->t('The @type temporary storage item with key "@key" in collection "@collection" has been deleted.', [
          '@type' => $tsk_entity->type,
          '@key' => $tsk_entity->key,
          '@collection' => $tsk_entity->collection,
        ]));
      }
    }
    catch (TempstoreKeyNotFoundException $e) {
      $this->messenger()->addWarning($this->t('The temporary storage item was not found: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (InvalidTempstoreTypeException $e) {
      $this->messenger()->addError($this->t('Invalid configuration: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (TskException $e) {
      $this->messenger()->addError($this->t('Failed to delete temporary storage: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (\InvalidArgumentException $e) {
      $this->messenger()->addError($this->t('Invalid parameters: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('An unexpected error occurred: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    return $this->redirect('entity.tsk_entity');
  }

  /**
   * Delete all specified temporary storages.
   */
  public function killAll() {
    try {
      $this->tskAdminService->killAll();
      $this->messenger()->addStatus($this->t('All specified temporary storages have been processed successfully.'));
    }
    catch (TskException $e) {
      $this->messenger()->addError($this->t('Some temporary storages failed to delete: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('An unexpected error occurred while processing temporary storages: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    return $this->redirect('entity.tsk_entity');
  }

  /**
   * Displays a list of all temporary storage collections.
   *
   * @return array
   *   A render array for the temporary storage list.
   */
  public function listTemporaryStorages() {
    try {
      // Query for temporary storage collections from key_value_expire table.
      $query = $this->database->select('key_value_expire', 'kve')
        ->fields('kve', ['collection', 'name', 'expire'])
        ->condition('collection', 'tempstore.%', 'LIKE')
        ->orderBy('collection')
        ->orderBy('name');

      $results = $query->execute()->fetchAll();
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Database error while retrieving temporary storage collections: @message', [
        '@message' => $e->getMessage(),
      ]));
      return [
        '#markup' => '<p>' . $this->t('Unable to retrieve temporary storage collections due to a database error.') . '</p>',
      ];
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Unexpected error while retrieving temporary storage collections: @message', [
        '@message' => $e->getMessage(),
      ]));
      return [
        '#markup' => '<p>' . $this->t('Unable to retrieve temporary storage collections.') . '</p>',
      ];
    }

    // Group results by collection.
    $collections = [];
    foreach ($results as $row) {
      // Extract collection type and name from the collection string.
      // Format: "tempstore.{type}.{collection_name}".
      $collection_parts = explode('.', $row->collection);
      if (count($collection_parts) >= 3) {
        // Type: private or shared.
        $type = $collection_parts[1];
        $collection_name = implode('.', array_slice($collection_parts, 2));

        if (!isset($collections[$collection_name])) {
          $collections[$collection_name] = [
            'name' => $collection_name,
            'types' => [],
            'items' => [],
          ];
        }

        if (!in_array($type, $collections[$collection_name]['types'])) {
          $collections[$collection_name]['types'][] = $type;
        }

        $collections[$collection_name]['items'][] = [
          'type' => $type,
          'key' => $row->name,
          'expire' => $row->expire,
          'expire_formatted' => $this->dateFormatter->format($row->expire),
        ];
      }
    }

    // Build the table.
    $header = [
      $this->t('Collection'),
      $this->t('Type(s)'),
      $this->t('Items Count'),
      $this->t('Sample Keys'),
      $this->t('Operations'),
    ];

    $rows = [];
    foreach ($collections as $collection) {
      $types_list = implode(', ', $collection['types']);
      $items_count = count($collection['items']);

      // Show first few keys as samples.
      $sample_keys = array_slice(array_column($collection['items'], 'key'), 0, 3);
      $sample_keys_display = implode(', ', $sample_keys);
      if ($items_count > 3) {
        $sample_keys_display .= '...';
      }

      $operations = [];

      // Add operations for each type found in this collection.
      foreach ($collection['types'] as $type) {
        $operations[] = [
          '#type' => 'link',
          '#title' => $this->t('Kill all @type', ['@type' => $type]),
          '#url' => Url::fromRoute('tsk_admin.kill_collection', [
            'collection' => $collection['name'],
            'type' => $type,
          ]),
          '#attributes' => [
            'class' => ['button', 'button--small', 'button--danger'],
            'onclick' => 'return confirm("' . $this->t('Are you sure you want to delete all @type items in collection "@collection"?', [
              '@type' => $type,
              '@collection' => $collection['name'],
            ]) . '")',
          ],
        ];
      }

      $rows[] = [
        'collection' => $collection['name'],
        'types' => $types_list,
        'count' => $items_count,
        'sample_keys' => $sample_keys_display,
        'operations' => [
          'data' => [
            '#theme' => 'item_list',
            '#items' => $operations,
            '#attributes' => ['class' => ['links', 'inline']],
          ],
        ],
      ];
    }

    $build = [];

    if (empty($rows)) {
      $build['empty'] = [
        '#markup' => '<p>' . $this->t('No temporary storage collections found.') . '</p>',
      ];
    }
    else {
      $build['description'] = [
        '#markup' => '<p>' . $this->t('This page lists all temporary storage collections currently in the system. Temporary storages are used to store temporary data that expires after a certain time.') . '</p>',
      ];

      $build['table'] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#empty' => $this->t('No temporary storage collections found.'),
      ];

      // Add a details section with individual items for debugging.
      $build['details'] = [
        '#type' => 'details',
        '#title' => $this->t('Individual Items'),
        '#description' => $this->t('Detailed list of all temporary storage items.'),
        '#open' => FALSE,
      ];

      $detail_header = [
        $this->t('Collection'),
        $this->t('Type'),
        $this->t('Key'),
        $this->t('Expires'),
        $this->t('Operations'),
      ];

      $detail_rows = [];
      foreach ($collections as $collection) {
        foreach ($collection['items'] as $item) {
          $detail_rows[] = [
            $collection['name'],
            $item['type'],
            $item['key'],
            $item['expire_formatted'],
            [
              'data' => [
                '#type' => 'link',
                '#title' => $this->t('Kill'),
                '#url' => Url::fromRoute('tsk_admin.kill_item', [
                  'collection' => $collection['name'],
                  'type' => $item['type'],
                  'key' => $item['key'],
                ]),
                '#attributes' => [
                  'class' => ['button', 'button--small', 'button--danger'],
                  'onclick' => 'return confirm("' . $this->t('Are you sure you want to delete the temporary storage item "@key"?', ['@key' => $item['key']]) . '")',
                ],
              ],
            ],
          ];
        }
      }

      $build['details']['items_table'] = [
        '#type' => 'table',
        '#header' => $detail_header,
        '#rows' => $detail_rows,
        '#empty' => $this->t('No temporary storage items found.'),
      ];
    }

    return $build;
  }

  /**
   * Kills all temporary storage items in a collection.
   *
   * @param string $collection
   *   The collection name.
   * @param string $type
   *   The storage type (private or shared).
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response.
   */
  public function killCollection($collection, $type) {
    // Validate input parameters.
    if (empty($collection)) {
      $this->messenger()->addError($this->t('Collection name cannot be empty.'));
      return $this->redirect('tsk_admin.list');
    }

    if (!in_array($type, ['private', 'shared'], TRUE)) {
      $this->messenger()->addError($this->t('Invalid storage type "@type". Only "private" and "shared" are supported.', [
        '@type' => $type,
      ]));
      return $this->redirect('tsk_admin.list');
    }

    try {
      $this->tskService->kill($collection, $type, TRUE);
      $this->messenger()->addStatus($this->t('All @type temporary storage items in collection "@collection" have been deleted.', [
        '@type' => $type,
        '@collection' => $collection,
      ]));
    }
    catch (TempstoreKeyNotFoundException $e) {
      $this->messenger()->addWarning($this->t('No temporary storage items were found: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (InvalidTempstoreTypeException $e) {
      $this->messenger()->addError($this->t('Invalid storage type: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (TskException $e) {
      $this->messenger()->addError($this->t('Failed to delete temporary storage collection: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (\InvalidArgumentException $e) {
      $this->messenger()->addError($this->t('Invalid parameters: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Unexpected error while deleting temporary storage collection: @message', [
        '@message' => $e->getMessage(),
      ]));
    }

    return $this->redirect('tsk_admin.list');
  }

  /**
   * Kills a specific temporary storage item.
   *
   * @param string $collection
   *   The collection name.
   * @param string $type
   *   The storage type (private or shared).
   * @param string $key
   *   The specific item key.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response.
   */
  public function killItem($collection, $type, $key) {
    // Validate input parameters.
    if (empty($collection)) {
      $this->messenger()->addError($this->t('Collection name cannot be empty.'));
      return $this->redirect('tsk_admin.list');
    }

    if (empty($key)) {
      $this->messenger()->addError($this->t('Storage key cannot be empty.'));
      return $this->redirect('tsk_admin.list');
    }

    if (!in_array($type, ['private', 'shared'], TRUE)) {
      $this->messenger()->addError($this->t('Invalid storage type "@type". Only "private" and "shared" are supported.', [
        '@type' => $type,
      ]));
      return $this->redirect('tsk_admin.list');
    }

    try {
      $this->tskService->kill($collection, $type, FALSE, $key);
      $this->messenger()->addStatus($this->t('Temporary storage item "@key" in @type collection "@collection" has been deleted.', [
        '@key' => $key,
        '@type' => $type,
        '@collection' => $collection,
      ]));
    }
    catch (TempstoreKeyNotFoundException $e) {
      $this->messenger()->addWarning($this->t('The temporary storage item was not found: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (InvalidTempstoreTypeException $e) {
      $this->messenger()->addError($this->t('Invalid storage type: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (TskException $e) {
      $this->messenger()->addError($this->t('Failed to delete temporary storage item: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (\InvalidArgumentException $e) {
      $this->messenger()->addError($this->t('Invalid parameters: @message', [
        '@message' => $e->getMessage(),
      ]));
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Unexpected error while deleting temporary storage item: @message', [
        '@message' => $e->getMessage(),
      ]));
    }

    return $this->redirect('tsk_admin.list');
  }

}
