<?php

namespace Drupal\tsk_admin\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * TSK entity form for the Temporary Storage Killer.
 */
class TskEntityForm extends EntityForm {

  /**
   * The database connection to be used.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs a new TskEntityForm.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection to be used.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
      // Load the service required to construct this class.
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $tsk_entity = $this->entity;

    // Prepare the form.
    $form = parent::form($form, $form_state);

    // Attach the custom library.
    $form['#attached']['library'][] = 'tsk_admin/entityForm';

    $form['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Tempstore type'),
      '#default_value' => $tsk_entity->type,
      '#description' => $this->t('Type of the tempstore.'),
      '#options' => [
        'private' => $this->t('private'),
        'shared' => $this->t('shared'),
      ],
    ];

    $form['collection'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Collection'),
      '#size' => 60,
      '#maxlength' => 255,
      '#required' => TRUE,
      '#default_value' => $tsk_entity->collection,
      '#description' => $this->t('The collection name to use for the key/value store. This is typically a shared namespace or module name, e.g. views, entity, etc.'),
    ];
    $form['id'] = [
      '#type' => 'machine_name',
      '#disabled' => !$tsk_entity->isNew(),
      '#default_value' => $tsk_entity->id(),
      '#machine_name' => [
        'exists' => [$this, 'exists'],
        'source' => ['collection'],
      ],
    ];

    $form['kill_all'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Kill all'),
      '#default_value' => $tsk_entity->kill_all,
      '#description' => $this->t('Deletes all stored key/value pairs from the collection.'),
    ];
    $form['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Key'),
      '#default_value' => $tsk_entity->key,
      '#description' => $this->t('The key of the stored data.'),
      '#states' => [
        // Only show this field when the 'kill_all' checkbox is disabled.
        'visible' => [
          ':input[name="kill_all"]' => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    if (!$tsk_entity->isNew()) {
      try {
        $query = $this->database->select('key_value_expire', 'kve');
        $query->fields('kve', ['collection', 'name']);
        $query->condition('kve.collection', $query->escapeLike('tempstore.') . '%', 'LIKE');
        $result = $query->execute()->fetchAll();

        $form['storages'] = [
          '#type' => 'details',
          '#title' => $this->t('List of temporary storages from the collection'),
          '#description' => $this->t('This table shows all temporary storages in the database from the collection: @collection', ['@collection' => $tsk_entity->collection ?? '']),
        ];

        $form['storages']['table'] = [
          '#type' => 'table',
          '#header' => [$this->t('Type'), $this->t('Collection'), $this->t('Key')],
          '#sticky' => TRUE,
          '#empty' => $this->t('There are no temporary storages in the database'),
        ];
        $remove_from_collection = ['tempstore.private.', 'tempstore.shared.'];
        foreach ($result as $i => $row) {
          $type = str_contains($row->collection, '.private.') ? $this->t('private') : $this->t('shared');
          $form['storages']['table'][$i]['type'] = [
            '#type' => 'html_tag',
            '#tag' => 'pre',
            '#value' => $type,
          ];
          $form['storages']['table'][$i]['collection'] = [
            '#type' => 'html_tag',
            '#tag' => 'pre',
            '#value' => str_replace($remove_from_collection, '', $row->collection),
          ];
          $form['storages']['table'][$i]['key'] = [
            '#type' => 'html_tag',
            '#tag' => 'pre',
            '#value' => $row->name,
          ];
        }
      }
      catch (\Exception $e) {
        $this->messenger()->addWarning($this->t('Unable to retrieve temporary storage information due to a database error: @message', [
          '@message' => $e->getMessage(),
        ]));

        $form['storages'] = [
          '#type' => 'details',
          '#title' => $this->t('List of temporary storages from the collection'),
          '#description' => $this->t('Unable to load temporary storage information.'),
        ];

        $form['storages']['error'] = [
          '#markup' => '<p>' . $this->t('Database error prevented loading the storage list.') . '</p>',
        ];
      }
      catch (\Exception $e) {
        $this->messenger()->addWarning($this->t('Unexpected error while retrieving temporary storage information: @message', [
          '@message' => $e->getMessage(),
        ]));

        $form['storages'] = [
          '#type' => 'details',
          '#title' => $this->t('List of temporary storages from the collection'),
          '#description' => $this->t('Unable to load temporary storage information.'),
        ];

        $form['storages']['error'] = [
          '#markup' => '<p>' . $this->t('An error occurred while loading the storage list.') . '</p>',
        ];
      }
    }

    // Return the form.
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $collection = trim($form_state->getValue('collection'));
    $type = $form_state->getValue('type');
    $kill_all = $form_state->getValue('kill_all');
    $key = trim($form_state->getValue('key'));

    // Validate collection name.
    if (empty($collection)) {
      $form_state->setErrorByName('collection', $this->t('Collection name cannot be empty.'));
    }
    elseif (strlen($collection) > 255) {
      $form_state->setErrorByName('collection', $this->t('Collection name cannot exceed 255 characters.'));
    }
    elseif (preg_match('/[^a-zA-Z0-9_.-]/', $collection)) {
      $form_state->setErrorByName('collection', $this->t('Collection name can only contain letters, numbers, dots, hyphens, and underscores.'));
    }

    // Validate type.
    if (!in_array($type, ['private', 'shared'], TRUE)) {
      $form_state->setErrorByName('type', $this->t('Invalid tempstore type. Must be either "private" or "shared".'));
    }

    // Validate key when not killing all.
    if (!$kill_all && empty($key)) {
      $form_state->setErrorByName('key', $this->t('Key cannot be empty when "Kill all" is not selected.'));
    }
    elseif (!$kill_all && strlen($key) > 128) {
      $form_state->setErrorByName('key', $this->t('Key cannot exceed 128 characters.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    try {
      $value = parent::save($form, $form_state);

      $this->messenger()->addStatus($this->t('The entity %id was saved successfully.', ['%id' => $this->entity->id]));

      // Redirect to the proper URL.
      $form_state->setRedirect('entity.tsk_entity');

      return $value;
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Failed to save the entity: @message', [
        '@message' => $e->getMessage(),
      ]));

      // Don't redirect on error, stay on the form.
      $form_state->setRebuild(TRUE);
      return FALSE;
    }
  }

  /**
   * Checks if the TSK entity exists.
   *
   * @param string $id
   *   Machine name (ID) of the TSK entity.
   *
   * @return bool
   *   Returns true, if the TSK entity exist.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function exists($id) {
    try {
      $storage = $this->entityTypeManager->getStorage('tsk_entity');
      $tsk_entity = $storage->getQuery()->accessCheck(FALSE)->condition('id', $id)->execute();
      return (bool) $tsk_entity;
    }
    catch (\Exception $e) {
      // Log the error but don't fail the check - assume entity doesn't exist.
      $this->logger('tsk_admin')->error('Error checking if TSK entity exists: @message', [
        '@message' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

}
