<?php

namespace Drupal\tsk_admin\Services;

use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\tsk\Exception\TskException;
use Drupal\tsk\Services\TskServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides methods to delete temporary storages.
 *
 * @package Drupal\tsk
 */
class TskAdminService implements TskAdminServiceInterface {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Temporary Storage Killer service.
   *
   * @var \Drupal\tsk\Services\TskServiceInterface
   */
  protected $tskService;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a TskService object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\tsk\Services\TskServiceInterface $tsk_service
   *   Temporary Storage Killer service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, TskServiceInterface $tsk_service, LoggerChannelFactoryInterface $logger_factory) {
    $this->entityTypeManager = $entity_type_manager;
    $this->tskService = $tsk_service;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('tsk.service'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\tsk\Exception\TskException
   *   When one or more temporary storage deletions fail.
   */
  public function killAll(): void {
    $failed_entities = [];
    $success_count = 0;

    try {
      $tsk_entities = $this->entityTypeManager->getStorage('tsk_entity')->loadMultiple();
    }
    catch (EntityStorageException $e) {
      $this->loggerFactory->get('tsk_admin')->error('Failed to load TSK entities: @message', [
        '@message' => $e->getMessage(),
      ]);
      throw new TskException('Failed to load TSK configuration entities.', 0, $e);
    }

    if (empty($tsk_entities)) {
      $this->loggerFactory->get('tsk_admin')->info('No TSK entities found to process.');
      return;
    }

    foreach ($tsk_entities as $tsk_entity) {
      try {
        $key = $tsk_entity->key ?? '';
        $this->tskService->kill($tsk_entity->collection, $tsk_entity->type, $tsk_entity->kill_all, $key);
        $success_count++;
        $this->loggerFactory->get('tsk_admin')->info('Successfully processed TSK entity "@id" for collection "@collection".', [
          '@id' => $tsk_entity->id(),
          '@collection' => $tsk_entity->collection,
        ]);
      }
      catch (TskException $e) {
        $failed_entities[] = [
          'entity_id' => $tsk_entity->id(),
          'collection' => $tsk_entity->collection,
          'type' => $tsk_entity->type,
          'error' => $e->getMessage(),
        ];
        $this->loggerFactory->get('tsk_admin')->error('Failed to process TSK entity "@id" for collection "@collection": @message', [
          '@id' => $tsk_entity->id(),
          '@collection' => $tsk_entity->collection,
          '@message' => $e->getMessage(),
        ]);
      }
      catch (\Exception $e) {
        $failed_entities[] = [
          'entity_id' => $tsk_entity->id(),
          'collection' => $tsk_entity->collection,
          'type' => $tsk_entity->type,
          'error' => $e->getMessage(),
        ];
        $this->loggerFactory->get('tsk_admin')->error('Unexpected error processing TSK entity "@id" for collection "@collection": @message', [
          '@id' => $tsk_entity->id(),
          '@collection' => $tsk_entity->collection,
          '@message' => $e->getMessage(),
        ]);
      }
    }

    // Log summary.
    $total_entities = count($tsk_entities);
    $failed_count = count($failed_entities);

    $this->loggerFactory->get('tsk_admin')->info('Processed @total TSK entities: @success successful, @failed failed.', [
      '@total' => $total_entities,
      '@success' => $success_count,
      '@failed' => $failed_count,
    ]);

    // If there were failures, throw an exception with details.
    if (!empty($failed_entities)) {
      $error_details = [];
      foreach ($failed_entities as $failure) {
        $error_details[] = sprintf('Entity %s (collection: %s, type: %s): %s',
          $failure['entity_id'],
          $failure['collection'],
          $failure['type'],
          $failure['error']
        );
      }

      throw new TskException(sprintf('Failed to process %d out of %d TSK entities: %s',
        $failed_count,
        $total_entities,
        implode('; ', $error_details)
      ));
    }
  }

}
