<?php

namespace Drupal\Tests\tsk_admin\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\tsk_admin\Entity\TskEntity;
use Drush\TestTraits\DrushTestTrait;

/**
 * Integration tests for TskAdminCommands Drush integration.
 *
 * @group tsk_admin
 * @group drush
 *
 * @coversDefaultClass \Drupal\tsk_admin\Commands\TskAdminCommands
 */
class TskAdminCommandsTest extends BrowserTestBase {

  use DrushTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['tsk', 'tsk_admin'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Private tempstore service.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $privateTempstore;

  /**
   * Shared tempstore service.
   *
   * @var \Drupal\Core\TempStore\SharedTempStoreFactory
   */
  protected $sharedTempstore;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->privateTempstore = $this->container->get('tempstore.private');
    $this->sharedTempstore = $this->container->get('tempstore.shared');
  }

  /**
   * Tests the temp-store-killer:all command with no TSK entities.
   *
   * @covers ::killAll
   */
  public function testKillAllWithNoEntities(): void {
    // Ensure no TSK entities exist.
    $storage = $this->container->get('entity_type.manager')->getStorage('tsk_entity');
    $entities = $storage->loadMultiple();
    $storage->delete($entities);

    // Execute the killAll command.
    $this->drush('temp-store-killer:all', []);

    // Command should succeed with no entities to process.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();
    $combined_output = $output . $error_output;

    $this->assertThat(
      $combined_output,
      $this->logicalOr(
        $this->stringContains('Successfully processed all configured temporary storage entities'),
        $this->isEmpty()
      )
    );
  }

  /**
   * Tests the temp-store-killer:all command with valid TSK entities.
   *
   * @covers ::killAll
   */
  public function testKillAllWithValidEntities(): void {
    // Create test data in temporary storages.
    $private_store = $this->privateTempstore->get('test_collection_private');
    $private_store->set('private_key1', 'private_value1');
    $private_store->set('private_key2', 'private_value2');

    $shared_store = $this->sharedTempstore->get('test_collection_shared');
    $shared_store->set('shared_key1', 'shared_value1');
    $shared_store->set('shared_key2', 'shared_value2');

    // Verify test data exists.
    $this->assertEquals('private_value1', $private_store->get('private_key1'));
    $this->assertEquals('private_value2', $private_store->get('private_key2'));
    $this->assertEquals('shared_value1', $shared_store->get('shared_key1'));
    $this->assertEquals('shared_value2', $shared_store->get('shared_key2'));

    // Create TSK entities to process the data.
    $entity1 = TskEntity::create([
      'id' => 'test_private_entity',
      'type' => 'private',
      'collection' => 'test_collection_private',
      'kill_all' => TRUE,
    ]);
    $entity1->save();

    $entity2 = TskEntity::create([
      'id' => 'test_shared_entity',
      'type' => 'shared',
      'collection' => 'test_collection_shared',
      'kill_all' => TRUE,
    ]);
    $entity2->save();

    // Execute the killAll command.
    $this->drush('temp-store-killer:all', []);

    // Verify command succeeded.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();
    $combined_output = $output . $error_output;

    $this->assertThat(
      $combined_output,
      $this->logicalOr(
        $this->stringContains('Successfully processed all configured temporary storage entities'),
        $this->isEmpty()
      )
    );

    // Most importantly, verify the data was actually deleted.
    $this->assertNull($private_store->get('private_key1'));
    $this->assertNull($private_store->get('private_key2'));
    $this->assertNull($shared_store->get('shared_key1'));
    $this->assertNull($shared_store->get('shared_key2'));
  }

  /**
   * Tests the temp-store-killer:all command with specific key entities.
   *
   * @covers ::killAll
   */
  public function testKillAllWithSpecificKeyEntities(): void {
    // Create test data in private tempstore.
    $private_store = $this->privateTempstore->get('specific_key_collection');
    $private_store->set('keep_key', 'keep_value');
    $private_store->set('delete_key', 'delete_value');

    // Verify test data exists.
    $this->assertEquals('keep_value', $private_store->get('keep_key'));
    $this->assertEquals('delete_value', $private_store->get('delete_key'));

    // Get the owner ID for private tempstore key format.
    $session = $this->container->get('request_stack')->getSession();
    if (!$session->isStarted()) {
      $session->start();
    }
    $owner = $session->get('core.tempstore.private.owner') ?: 'anonymous';
    $full_delete_key = $owner . ':delete_key';

    // Create TSK entity to delete only specific key.
    $entity = TskEntity::create([
      'id' => 'specific_key_entity',
      'type' => 'private',
      'collection' => 'specific_key_collection',
      'kill_all' => FALSE,
      'key' => $full_delete_key,
    ]);
    $entity->save();

    // Execute the killAll command.
    $this->drush('temp-store-killer:all', []);

    // Verify command succeeded.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();
    $combined_output = $output . $error_output;

    $this->assertThat(
      $combined_output,
      $this->logicalOr(
        $this->stringContains('Successfully processed all configured temporary storage entities'),
        $this->isEmpty()
      )
    );

    // Verify only the specific key was deleted.
    $this->assertEquals('keep_value', $private_store->get('keep_key'));
    $this->assertNull($private_store->get('delete_key'));
  }

  /**
   * Tests the tska command with mixed valid and invalid entities.
   *
   * @covers ::killAll
   */
  public function testKillAllWithMixedValidInvalidEntities(): void {
    // Create test data for valid entity.
    $valid_store = $this->privateTempstore->get('valid_collection');
    $valid_store->set('valid_key', 'valid_value');

    // Verify test data exists.
    $this->assertEquals('valid_value', $valid_store->get('valid_key'));

    // Create a valid TSK entity.
    $valid_entity = TskEntity::create([
      'id' => 'valid_entity',
      'type' => 'private',
      'collection' => 'valid_collection',
      'kill_all' => TRUE,
    ]);
    $valid_entity->save();

    // Create an entity with invalid/non-existent collection.
    $invalid_entity = TskEntity::create([
      'id' => 'invalid_entity',
      'type' => 'private',
      'collection' => 'nonexistent_collection',
      'kill_all' => TRUE,
    ]);
    $invalid_entity->save();

    // Execute the killAll command.
    $this->drush('temp-store-killer:all', []);

    // Command should handle mixed scenarios - either succeed with warnings
    // or show error about partial failures.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();
    $combined_output = $output . $error_output;

    $this->assertThat(
      $combined_output,
      $this->logicalOr(
        $this->stringContains('Successfully processed all configured temporary storage entities'),
        $this->stringContains('Some temporary storage entities failed to process'),
        $this->isEmpty()
      )
    );

    // The valid entity should still have been processed successfully.
    $this->assertNull($valid_store->get('valid_key'));
  }

  /**
   * Tests the temp-store-killer:all command with different storage types.
   *
   * @covers ::killAll
   */
  public function testKillAllWithDifferentStorageTypes(): void {
    // Create test data in both storage types.
    $private_store = $this->privateTempstore->get('mixed_types_private');
    $private_store->set('private_item', 'private_data');

    $shared_store = $this->sharedTempstore->get('mixed_types_shared');
    $shared_store->set('shared_item', 'shared_data');

    // Verify test data exists.
    $this->assertEquals('private_data', $private_store->get('private_item'));
    $this->assertEquals('shared_data', $shared_store->get('shared_item'));

    // Create TSK entities for both storage types.
    $private_entity = TskEntity::create([
      'id' => 'mixed_private_entity',
      'type' => 'private',
      'collection' => 'mixed_types_private',
      'kill_all' => TRUE,
    ]);
    $private_entity->save();

    $shared_entity = TskEntity::create([
      'id' => 'mixed_shared_entity',
      'type' => 'shared',
      'collection' => 'mixed_types_shared',
      'kill_all' => TRUE,
    ]);
    $shared_entity->save();

    // Execute the killAll command.
    $this->drush('temp-store-killer:all', []);

    // Verify command succeeded.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();
    $combined_output = $output . $error_output;

    $this->assertThat(
      $combined_output,
      $this->logicalOr(
        $this->stringContains('Successfully processed all configured temporary storage entities'),
        $this->isEmpty()
      )
    );

    // Verify data from both storage types was deleted.
    $this->assertNull($private_store->get('private_item'));
    $this->assertNull($shared_store->get('shared_item'));
  }

  /**
   * Tests the tska alias for the temp-store-killer:all command.
   *
   * @covers ::killAll
   */
  public function testTskaAlias(): void {
    // Create test data.
    $test_store = $this->privateTempstore->get('alias_test_collection');
    $test_store->set('alias_key', 'alias_value');

    // Verify test data exists.
    $this->assertEquals('alias_value', $test_store->get('alias_key'));

    // Create TSK entity.
    $entity = TskEntity::create([
      'id' => 'alias_test_entity',
      'type' => 'private',
      'collection' => 'alias_test_collection',
      'kill_all' => TRUE,
    ]);
    $entity->save();

    // Use the 'tska' alias instead of the full command name.
    $this->drush('tska', []);

    // Verify the data was deleted (the main test).
    $this->assertNull($test_store->get('alias_key'));
  }

  /**
   * Tests the temp-store-killer:all command with non-existent collections.
   *
   * @covers ::killAll
   */
  public function testKillAllWithNonExistentCollections(): void {
    // Create TSK entities for non-existent collections.
    $entity1 = TskEntity::create([
      'id' => 'nonexistent_entity1',
      'type' => 'private',
      'collection' => 'completely_nonexistent_collection',
      'kill_all' => TRUE,
    ]);
    $entity1->save();

    $entity2 = TskEntity::create([
      'id' => 'nonexistent_entity2',
      'type' => 'shared',
      'collection' => 'another_nonexistent_collection',
      'kill_all' => TRUE,
    ]);
    $entity2->save();

    // Execute the killAll command.
    $this->drush('temp-store-killer:all', []);

    // Command should handle non-existent collections gracefully.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();
    $combined_output = $output . $error_output;

    $this->assertThat(
      $combined_output,
      $this->logicalOr(
        $this->stringContains('Successfully processed all configured temporary storage entities'),
        $this->stringContains('Some temporary storage entities failed to process'),
        $this->isEmpty()
      )
    );

    // Main requirement: command should not crash.
    $this->assertTrue(TRUE, 'Command completed without fatal errors');
  }

  /**
   * Tests comprehensive workflow with multiple collections and keys.
   *
   * @covers ::killAll
   */
  public function testKillAllComprehensiveWorkflow(): void {
    // Create complex test with multiple collections and mixed operations.
    // Collection 1: Private tempstore, delete all.
    $private_store1 = $this->privateTempstore->get('workflow_collection1');
    $private_store1->set('item1', 'value1');
    $private_store1->set('item2', 'value2');

    // Collection 2: Shared tempstore, delete all.
    $shared_store1 = $this->sharedTempstore->get('workflow_collection2');
    $shared_store1->set('shared_item1', 'shared_value1');
    $shared_store1->set('shared_item2', 'shared_value2');

    // Collection 3: Private tempstore, delete specific key.
    $private_store2 = $this->privateTempstore->get('workflow_collection3');
    $private_store2->set('keep_item', 'keep_value');
    $private_store2->set('delete_item', 'delete_value');

    // Verify initial data exists.
    $this->assertEquals('value1', $private_store1->get('item1'));
    $this->assertEquals('value2', $private_store1->get('item2'));
    $this->assertEquals('shared_value1', $shared_store1->get('shared_item1'));
    $this->assertEquals('shared_value2', $shared_store1->get('shared_item2'));
    $this->assertEquals('keep_value', $private_store2->get('keep_item'));
    $this->assertEquals('delete_value', $private_store2->get('delete_item'));

    // Create corresponding TSK entities.
    $entity1 = TskEntity::create([
      'id' => 'workflow_entity1',
      'type' => 'private',
      'collection' => 'workflow_collection1',
      'kill_all' => TRUE,
    ]);
    $entity1->save();

    $entity2 = TskEntity::create([
      'id' => 'workflow_entity2',
      'type' => 'shared',
      'collection' => 'workflow_collection2',
      'kill_all' => TRUE,
    ]);
    $entity2->save();

    // Get owner for specific key deletion.
    $session = $this->container->get('request_stack')->getSession();
    if (!$session->isStarted()) {
      $session->start();
    }
    $owner = $session->get('core.tempstore.private.owner') ?: 'anonymous';
    $full_delete_key = $owner . ':delete_item';

    $entity3 = TskEntity::create([
      'id' => 'workflow_entity3',
      'type' => 'private',
      'collection' => 'workflow_collection3',
      'kill_all' => FALSE,
      'key' => $full_delete_key,
    ]);
    $entity3->save();

    // Execute the killAll command.
    $this->drush('temp-store-killer:all', []);

    // Verify command completed.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();
    $combined_output = $output . $error_output;

    $this->assertThat(
      $combined_output,
      $this->logicalOr(
        $this->stringContains('Successfully processed all configured temporary storage entities'),
        $this->isEmpty()
      )
    );

    // Verify expected data deletions occurred.
    // Collection 1: All items should be deleted.
    $this->assertNull($private_store1->get('item1'));
    $this->assertNull($private_store1->get('item2'));

    // Collection 2: All items should be deleted.
    $this->assertNull($shared_store1->get('shared_item1'));
    $this->assertNull($shared_store1->get('shared_item2'));

    // Collection 3: Only specific item should be deleted.
    $this->assertEquals('keep_value', $private_store2->get('keep_item'));
    $this->assertNull($private_store2->get('delete_item'));
  }

  /**
   * Tests error handling for database/infrastructure issues.
   *
   * @covers ::killAll
   */
  public function testKillAllErrorHandling(): void {
    // Create TSK entities that might cause infrastructure issues.
    $entity = TskEntity::create([
      'id' => 'error_handling_entity',
      'type' => 'private',
      'collection' => 'error_test_collection',
      'kill_all' => TRUE,
    ]);
    $entity->save();

    // Execute the killAll command.
    $this->drush('temp-store-killer:all', []);

    // Command should handle any infrastructure errors gracefully.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();
    $combined_output = $output . $error_output;

    // Should either succeed or show handled error messages.
    $this->assertThat(
      $combined_output,
      $this->logicalOr(
        $this->stringContains('Successfully processed all configured temporary storage entities'),
        $this->stringContains('Some temporary storage entities failed to process'),
        $this->stringContains('Unexpected error while processing temporary storage entities'),
        $this->isEmpty()
      )
    );

    // Main requirement: command should not crash with unhandled exceptions.
    $this->assertTrue(TRUE, 'Command handled all scenarios without fatal errors');
  }

  /**
   * Tests special characters in collection names and keys.
   *
   * @covers ::killAll
   */
  public function testKillAllWithSpecialCharacters(): void {
    // Create test data with special characters.
    $special_collection = 'test-collection_with.dots@special';
    $special_store = $this->privateTempstore->get($special_collection);
    $special_store->set('special@key.test', 'special_value');

    // Verify test data exists.
    $this->assertEquals('special_value', $special_store->get('special@key.test'));

    // Create TSK entity with special characters.
    $entity = TskEntity::create([
      'id' => 'special_chars_entity',
      'type' => 'private',
      'collection' => $special_collection,
      'kill_all' => TRUE,
    ]);
    $entity->save();

    // Execute the killAll command.
    $this->drush('temp-store-killer:all', []);

    // Verify the command handled special characters correctly.
    $this->assertNull($special_store->get('special@key.test'));
  }

  /**
   * Tests verbose output functionality.
   *
   * @covers ::killAll
   */
  public function testKillAllVerboseOutput(): void {
    // Create test data.
    $verbose_store = $this->privateTempstore->get('verbose_test_collection');
    $verbose_store->set('verbose_key', 'verbose_value');

    // Create TSK entity.
    $entity = TskEntity::create([
      'id' => 'verbose_test_entity',
      'type' => 'private',
      'collection' => 'verbose_test_collection',
      'kill_all' => TRUE,
    ]);
    $entity->save();

    // Execute the killAll command with verbose flag.
    $this->drush('temp-store-killer:all', [], ['verbose' => TRUE]);

    // Main test: the data should be deleted regardless of verbose output.
    $this->assertNull($verbose_store->get('verbose_key'));
  }

}
