<?php

namespace Drupal\Tests\tsk_admin\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\tsk_admin\Entity\TskEntity;

/**
 * Test the TskEntityForm.
 *
 * @group tsk_admin
 * @group tsk
 *
 * @coversDefaultClass \Drupal\tsk_admin\Form\TskEntityForm
 */
class TskEntityFormTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['tsk', 'tsk_admin'];

  /**
   * Disable strict config schema checking for this test.
   *
   * @var bool
   */
  protected $strictConfigSchema = FALSE;

  /**
   * A user with permission to administer TSK.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create a user with administrative permissions.
    $this->adminUser = $this->drupalCreateUser([
      'administer tsk',
      'access administration pages',
    ]);
  }

  /**
   * Test accessing the TSK entity add form.
   *
   * @covers ::form
   */
  public function testAddFormAccess() {
    // Test anonymous access is denied.
    $this->drupalGet('/admin/config/development/tsk/add');
    $this->assertSession()->statusCodeEquals(403);

    // Test authenticated user without permission is denied.
    $user = $this->drupalCreateUser();
    $this->drupalLogin($user);
    $this->drupalGet('/admin/config/development/tsk/add');
    $this->assertSession()->statusCodeEquals(403);

    // Test authenticated user with permission can access.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/tsk/add');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Add TSK entity');
  }

  /**
   * Test the form structure and default values.
   *
   * @covers ::form
   */
  public function testFormStructure() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/tsk/add');

    // Check that required form fields are present.
    $this->assertSession()->fieldExists('type');
    $this->assertSession()->fieldExists('collection');
    $this->assertSession()->fieldExists('id');
    $this->assertSession()->fieldExists('kill_all');
    $this->assertSession()->fieldExists('key');

    // Check field types and default values.
    $this->assertSession()->optionExists('type', 'private');
    $this->assertSession()->optionExists('type', 'shared');
    $this->assertSession()->fieldValueEquals('type', 'private');
    $this->assertSession()->fieldValueEquals('kill_all', '1');

    // Check that collection field is required.
    $this->assertSession()->elementAttributeContains('css', 'input[name="collection"]', 'required', 'required');

    // Check that the key field has states (visibility controlled by kill_all).
    $this->assertSession()->elementExists('css', 'input[name="key"]');
  }

  /**
   * Test successful form submission for creating a new entity.
   *
   * @covers ::save
   * @covers ::validateForm
   */
  public function testCreateEntitySuccess() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/tsk/add');

    // Fill out the form with machine name.
    $edit = [
      'type' => 'private',
      'collection' => 'test_collection',
    // Machine name is required.
      'id' => 'test_collection',
      'kill_all' => '1',
    ];
    $this->submitForm($edit, 'Save');

    // Check that we're redirected to the listing page.
    $this->assertSession()->addressEquals('/admin/config/development/tsk');

    // Check for success message.
    $this->assertSession()->pageTextContains('saved successfully');

    // Verify the entity exists in the database.
    $entity = TskEntity::load('test_collection');
    $this->assertNotNull($entity);
    $this->assertEquals('private', $entity->type);
    $this->assertEquals('test_collection', $entity->collection);
    $this->assertTrue($entity->kill_all);
  }

  /**
   * Test creating entity with specific key (kill_all = FALSE).
   *
   * @covers ::save
   * @covers ::validateForm
   */
  public function testCreateEntityWithKey() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/tsk/add');

    // Fill out the form with specific key.
    $edit = [
      'type' => 'shared',
      'collection' => 'key_test_collection',
    // Machine name is required.
      'id' => 'key_test_collection',
      'kill_all' => '0',
      'key' => 'specific_key',
    ];
    $this->submitForm($edit, 'Save');

    // Check that entity was created successfully.
    $this->assertSession()->pageTextContains('saved successfully');

    // Verify the entity exists with correct values.
    $entity = TskEntity::load('key_test_collection');
    $this->assertNotNull($entity);
    $this->assertEquals('shared', $entity->type);
    $this->assertEquals('key_test_collection', $entity->collection);
    $this->assertFalse($entity->kill_all);
    $this->assertEquals('specific_key', $entity->key);
  }

  /**
   * Test form validation errors.
   *
   * @covers ::validateForm
   */
  public function testFormValidation() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/tsk/add');

    // Skip empty collection validation as it's handled by HTML5 required
    // attribute and not easily testable in functional tests without JavaScript
    // support. Skip collection name length validation (over 255 characters) as
    // it's blocked by HTML5 maxlength attribute. The server-side validation
    // exists but cannot be tested in functional tests without JavaScript
    // support. Test collection name character validation.
    $edit = [
      'type' => 'private',
      'collection' => 'invalid@collection#name',
      'id' => 'test_id3',
      'kill_all' => '1',
    ];
    $this->submitForm($edit, 'Save');
    $this->assertSession()->pageTextContains('Collection name can only contain letters, numbers, dots, hyphens, and underscores.');

    // Test key validation when kill_all is FALSE.
    $edit = [
      'type' => 'private',
      'collection' => 'valid_collection',
      'id' => 'test_id4',
      'kill_all' => '0',
      'key' => '',
    ];
    $this->submitForm($edit, 'Save');
    $this->assertSession()->pageTextContains('Key cannot be empty when "Kill all" is not selected.');
  }

  /**
   * Test editing an existing entity.
   *
   * @covers ::form
   * @covers ::save
   */
  public function testEditEntity() {
    // Create an entity first.
    $entity = TskEntity::create([
      'id' => 'edit_test',
      'type' => 'private',
      'collection' => 'edit_collection',
      'kill_all' => TRUE,
    ]);
    $entity->save();

    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/tsk/edit_test');

    // Check that form is populated with existing values.
    $this->assertSession()->fieldValueEquals('type', 'private');
    $this->assertSession()->fieldValueEquals('collection', 'edit_collection');
    $this->assertSession()->fieldValueEquals('kill_all', '1');

    // Check that ID field is disabled for existing entities.
    $this->assertSession()->elementAttributeContains('css', 'input[name="id"]', 'disabled', 'disabled');

    // Modify the entity.
    $edit = [
      'type' => 'shared',
      'collection' => 'modified_collection',
      'kill_all' => '0',
      'key' => 'modified_key',
    ];
    $this->submitForm($edit, 'Save');

    // Check success message.
    $this->assertSession()->pageTextContains('The entity edit_test was saved successfully.');

    // Verify changes were saved.
    $entity = TskEntity::load('edit_test');
    $this->assertEquals('shared', $entity->type);
    $this->assertEquals('modified_collection', $entity->collection);
    $this->assertFalse($entity->kill_all);
    $this->assertEquals('modified_key', $entity->key);
  }

  /**
   * Test the exists() method for machine name validation.
   *
   * @covers ::exists
   */
  public function testExistsMethod() {
    // Create an entity.
    $entity = TskEntity::create([
      'id' => 'existing_entity',
      'type' => 'private',
      'collection' => 'test_collection',
      'kill_all' => TRUE,
    ]);
    $entity->save();

    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/tsk/add');

    // Try to create an entity with the same ID.
    $edit = [
      'type' => 'private',
    // This will generate the same machine name.
      'collection' => 'existing_entity',
      'kill_all' => '1',
    ];

    // Fill in the collection field and trigger machine name generation.
    $this->getSession()->getPage()->fillField('collection', 'existing_entity');

    // The machine name field should show an error about the ID already
    // existing.
    // Note: The exact behavior depends on the JavaScript implementation.
    // For now, we'll test that we can't submit with duplicate ID.
    $edit['id'] = 'existing_entity';
    $this->submitForm($edit, 'Save');

    // Should get a validation error about duplicate entity.
    $this->assertSession()->pageTextContains('The machine-readable name is already in use');
  }

  /**
   * Test form display with temporary storage information for existing entities.
   *
   * @covers ::form
   */
  public function testFormWithStorageInfo() {
    // Create some temporary storage data.
    $store_private = \Drupal::service('tempstore.private')->get('storage_info_test');
    $store_private->set('test_key1', 'test_value1');

    $store_shared = \Drupal::service('tempstore.shared')->get('storage_info_test');
    $store_shared->set('shared_key1', 'shared_value1');

    // Create an entity.
    $entity = TskEntity::create([
      'id' => 'storage_info_entity',
      'type' => 'private',
      'collection' => 'storage_info_test',
      'kill_all' => TRUE,
    ]);
    $entity->save();

    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/tsk/storage_info_entity');

    // Check that the storage information section is present.
    $this->assertSession()->pageTextContains('List of temporary storages from the collection');
    $this->assertSession()->pageTextContains('This table shows all temporary storages in the database from the collection: storage_info_test');

    // The temporary storage table should be displayed.
    $this->assertSession()->elementExists('css', 'table');
    $this->assertSession()->pageTextContains('Type');
    $this->assertSession()->pageTextContains('Collection');
    $this->assertSession()->pageTextContains('Key');
  }

  /**
   * Test that the form handles database errors gracefully.
   *
   * @covers ::form
   */
  public function testFormWithDatabaseError() {
    // This test would ideally mock the database to throw an exception,
    // but in a functional test we can't easily do that.
    // We'll create an entity and verify the form loads correctly.
    $entity = TskEntity::create([
      'id' => 'db_error_test',
      'type' => 'private',
      'collection' => 'error_test',
      'kill_all' => TRUE,
    ]);
    $entity->save();

    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/tsk/db_error_test');

    // Form should load without errors.
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Tempstore type');
  }

  /**
   * Test form library attachment.
   *
   * @covers ::form
   */
  public function testFormLibraryAttachment() {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/tsk/add');

    // Check that the custom library is attached.
    // This would be visible in the HTML source as a script/CSS reference.
    // For a functional test, we can verify the form renders correctly.
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->fieldExists('collection');
  }

}
