<?php

namespace Drupal\Tests\tsk_admin\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\tsk\Exception\TskException;
use Drupal\tsk_admin\Entity\TskEntity;
use Drupal\tsk_admin\Services\TskAdminService;

/**
 * Test the TskAdminService.
 *
 * @group tsk_admin
 * @group tsk
 *
 * @coversDefaultClass \Drupal\tsk_admin\Services\TskAdminService
 */
class TskAdminServiceTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   *
   * Any modules added here will be loaded, along with anything in $modules in
   * parent classes.
   *
   * These modules are not installed, but their services and hooks are
   * available.
   *
   * @var string[]
   */
  protected static $modules = ['tsk', 'tsk_admin'];

  /**
   * Disable strict config schema checking for this test.
   *
   * @var bool
   */
  protected $strictConfigSchema = FALSE;

  /**
   * The TSK admin service.
   *
   * @var \Drupal\tsk_admin\Services\TskAdminServiceInterface
   */
  protected $tskAdminService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Install the configuration schema.
    $this->installConfig(['tsk_admin']);

    // Get the services.
    $this->tskAdminService = $this->container->get('tsk_admin.service');
    $this->entityTypeManager = $this->container->get('entity_type.manager');
  }

  /**
   * Test the TskAdminService service instantiation.
   *
   * @covers ::__construct
   */
  public function testServiceInstantiation() {
    $this->assertInstanceOf(
      'Drupal\tsk_admin\Services\TskAdminServiceInterface',
      $this->tskAdminService
    );
  }

  /**
   * Test killAll method with no TSK entities.
   *
   * @covers ::killAll
   */
  public function testKillAllWithNoEntities() {
    // Ensure no TSK entities exist.
    $storage = $this->entityTypeManager->getStorage('tsk_entity');
    $entities = $storage->loadMultiple();
    $this->assertEmpty($entities, 'No TSK entities should exist initially.');

    // Call killAll - should not throw an exception.
    $this->tskAdminService->killAll();

    // Verify it completes successfully.
    $this->assertTrue(TRUE, 'killAll completed successfully with no entities.');
  }

  /**
   * Test killAll method with valid TSK entities.
   *
   * @covers ::killAll
   */
  public function testKillAllWithValidEntities() {
    // Create some temporary storage data to delete.
    $store_private = \Drupal::service('tempstore.private')->get('test_collection');
    $store_private->set('test_key1', 'test_value1');
    $store_private->set('test_key2', 'test_value2');

    $store_shared = \Drupal::service('tempstore.shared')->get('test_collection');
    $store_shared->set('shared_key1', 'shared_value1');

    // Verify the data exists.
    $this->assertEquals('test_value1', $store_private->get('test_key1'));
    $this->assertEquals('test_value2', $store_private->get('test_key2'));
    $this->assertEquals('shared_value1', $store_shared->get('shared_key1'));

    // Create TSK entities for testing.
    $entity1 = TskEntity::create([
      'id' => 'test_entity_1',
      'type' => 'private',
      'collection' => 'test_collection',
      'kill_all' => TRUE,
    ]);
    $entity1->save();

    $entity2 = TskEntity::create([
      'id' => 'test_entity_2',
      'type' => 'shared',
      'collection' => 'test_collection',
      'kill_all' => TRUE,
    ]);
    $entity2->save();

    // Verify entities are created.
    $storage = $this->entityTypeManager->getStorage('tsk_entity');
    $entities = $storage->loadMultiple();
    $this->assertCount(2, $entities, 'Two TSK entities should exist.');

    // Call killAll.
    $this->tskAdminService->killAll();

    // Verify the temporary storage data has been deleted.
    $this->assertNull($store_private->get('test_key1'));
    $this->assertNull($store_private->get('test_key2'));
    $this->assertNull($store_shared->get('shared_key1'));
  }

  /**
   * Test killAll method with specific key deletion.
   *
   * @covers ::killAll
   */
  public function testKillAllWithSpecificKey() {
    // Create some temporary storage data.
    $store_private = \Drupal::service('tempstore.private')->get('specific_test');
    $store_private->set('keep_key', 'keep_value');
    $store_private->set('delete_key', 'delete_value');

    // Get the private key format (owner:key).
    $session = \Drupal::service('request_stack')->getSession();
    $owner = $session->get('core.tempstore.private.owner');
    $delete_key = $owner . ':delete_key';

    // Create TSK entity for specific key deletion.
    $entity = TskEntity::create([
      'id' => 'specific_key_test',
      'type' => 'private',
      'collection' => 'specific_test',
      'kill_all' => FALSE,
      'key' => $delete_key,
    ]);
    $entity->save();

    // Verify initial data.
    $this->assertEquals('keep_value', $store_private->get('keep_key'));
    $this->assertEquals('delete_value', $store_private->get('delete_key'));

    // Call killAll.
    $this->tskAdminService->killAll();

    // Verify only the specific key was deleted.
    $this->assertEquals('keep_value', $store_private->get('keep_key'));
    $this->assertNull($store_private->get('delete_key'));
  }

  /**
   * Test killAll method with mixed valid and invalid entities.
   *
   * @covers ::killAll
   */
  public function testKillAllWithMixedEntities() {
    // Create some temporary storage data.
    $store_private = \Drupal::service('tempstore.private')->get('mixed_test');
    $store_private->set('valid_key', 'valid_value');

    // Create a valid entity.
    $valid_entity = TskEntity::create([
      'id' => 'valid_entity',
      'type' => 'private',
      'collection' => 'mixed_test',
      'kill_all' => TRUE,
    ]);
    $valid_entity->save();

    // Create an entity with invalid collection (non-existent).
    $invalid_entity = TskEntity::create([
      'id' => 'invalid_entity',
      'type' => 'private',
      'collection' => 'nonexistent_collection',
      'kill_all' => TRUE,
    ]);
    $invalid_entity->save();

    // Verify initial data.
    $this->assertEquals('valid_value', $store_private->get('valid_key'));

    // Call killAll - should handle both valid and invalid entities.
    try {
      $this->tskAdminService->killAll();
      // If no exception is thrown, that's also valid (depends on
      // implementation).
    }
    catch (TskException $e) {
      // This is expected when there are failures.
      $this->assertStringContainsString('Failed to process', $e->getMessage());
    }

    // The valid entity should still have processed successfully.
    $this->assertNull($store_private->get('valid_key'));
  }

  /**
   * Test killAll method with shared tempstore.
   *
   * @covers ::killAll
   */
  public function testKillAllWithSharedTempstore() {
    // Create shared temporary storage data.
    $store_shared = \Drupal::service('tempstore.shared')->get('shared_test');
    $store_shared->set('shared_key1', 'shared_value1');
    $store_shared->set('shared_key2', 'shared_value2');

    // Create TSK entity for shared tempstore.
    $entity = TskEntity::create([
      'id' => 'shared_test_entity',
      'type' => 'shared',
      'collection' => 'shared_test',
      'kill_all' => TRUE,
    ]);
    $entity->save();

    // Verify initial data.
    $this->assertEquals('shared_value1', $store_shared->get('shared_key1'));
    $this->assertEquals('shared_value2', $store_shared->get('shared_key2'));

    // Call killAll.
    $this->tskAdminService->killAll();

    // Verify all shared data was deleted.
    $this->assertNull($store_shared->get('shared_key1'));
    $this->assertNull($store_shared->get('shared_key2'));
  }

  /**
   * Test service dependencies are correctly injected.
   *
   * @covers ::__construct
   * @covers ::create
   */
  public function testServiceDependencies() {
    // Test that the service can be created through the container.
    $service = $this->container->get('tsk_admin.service');
    $this->assertInstanceOf(
      'Drupal\tsk_admin\Services\TskAdminServiceInterface',
      $service
    );

    // Test that the static create method works.
    $created_service = TskAdminService::create($this->container);
    $this->assertInstanceOf(
      'Drupal\tsk_admin\Services\TskAdminService',
      $created_service
    );
  }

}
