<?php

namespace Drupal\tsk\Commands;

use Drush\Commands\DrushCommands;
use Drupal\tsk\Exception\InvalidTempstoreTypeException;
use Drupal\tsk\Exception\TempstoreKeyNotFoundException;
use Drupal\tsk\Exception\TskException;
use Drupal\tsk\Services\TskServiceInterface;

/**
 * Drush integration for the module Temporary Storages Killer.
 */
class TskCommands extends DrushCommands {

  /**
   * TskService.
   *
   * @var \Drupal\tsk\Services\TskServiceInterface
   */
  protected $tskService;

  /**
   * TskCommands constructor.
   *
   * @param \Drupal\tsk\Services\TskServiceInterface $tsk_service
   *   Temporary Storage Killer service.
   */
  public function __construct(TskServiceInterface $tsk_service) {
    $this->tskService = $tsk_service;
  }

  /**
   * Kills a temporary storage.
   *
   * @param string $type
   *   Type of the collection: private / shared.
   * @param string $collection
   *   The collection name to use for this key/value store.
   * @param string $key
   *   The key of the stored data.
   *
   * @command temp-store-killer
   * @aliases tsk
   *
   * @usage drush temp-store-killer type collection key
   *   Kills a temporary storage.
   */
  public function kill(string $type, string $collection, string $key = '') {
    // Validate input parameters.
    if (empty($type)) {
      $this->logger()->error('Storage type cannot be empty. Use "private" or "shared".');
      return;
    }

    if (empty($collection)) {
      $this->logger()->error('Collection name cannot be empty.');
      return;
    }

    if (!in_array($type, ['private', 'shared'], TRUE)) {
      $this->logger()->error('Invalid storage type "{type}". Only "private" and "shared" are supported.', ['type' => $type]);
      return;
    }

    $kill_all = empty($key);

    try {
      $this->tskService->kill($collection, $type, $kill_all, $key);

      if ($kill_all) {
        $this->logger()->success('Successfully deleted all {type} temporary storage items in collection "{collection}".', [
          'type' => $type,
          'collection' => $collection,
        ]);
      }
      else {
        $this->logger()->success('Successfully deleted {type} temporary storage item "{key}" in collection "{collection}".', [
          'type' => $type,
          'key' => $key,
          'collection' => $collection,
        ]);
      }
    }
    catch (TempstoreKeyNotFoundException $e) {
      $this->logger()->warning('Temporary storage item not found: {message}', ['message' => $e->getMessage()]);
    }
    catch (InvalidTempstoreTypeException $e) {
      $this->logger()->error('Invalid storage type: {message}', ['message' => $e->getMessage()]);
    }
    catch (TskException $e) {
      $this->logger()->error('Failed to delete temporary storage: {message}', ['message' => $e->getMessage()]);
    }
    catch (\InvalidArgumentException $e) {
      $this->logger()->error('Invalid parameters: {message}', ['message' => $e->getMessage()]);
    }
    catch (\Exception $e) {
      $this->logger()->error('Unexpected error: {message}', ['message' => $e->getMessage()]);
    }
  }

}
