<?php

namespace Drupal\tsk\Services;

use Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\TempStore\SharedTempStoreFactory;
use Drupal\tsk\Exception\InvalidTempstoreTypeException;
use Drupal\tsk\Exception\TempstoreDatabaseException;
use Drupal\tsk\Exception\TempstoreKeyNotFoundException;

/**
 * Provides methods to delete temporary storages.
 *
 * @package Drupal\tsk
 */
class TskService implements TskServiceInterface {

  /**
   * The storage factory creating the backend to store the data.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface
   */
  protected $storageFactory;

  /**
   * The private tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $privateTempStoreFactory;

  /**
   * The shared tempstore factory.
   *
   * @var \Drupal\Core\TempStore\SharedTempStoreFactory
   */
  protected $sharedTempStoreFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a TskService object.
   *
   * @param \Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface $storage_factory
   *   The key/value store factory.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $private_tempstore_factory
   *   The private tempstore factory.
   * @param \Drupal\Core\TempStore\SharedTempStoreFactory $shared_tempstore_factory
   *   The shared tempstore factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(KeyValueExpirableFactoryInterface $storage_factory, PrivateTempStoreFactory $private_tempstore_factory, SharedTempStoreFactory $shared_tempstore_factory, LoggerChannelFactoryInterface $logger_factory) {
    $this->storageFactory = $storage_factory;
    $this->privateTempStoreFactory = $private_tempstore_factory;
    $this->sharedTempStoreFactory = $shared_tempstore_factory;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function kill(string $collection, string $type = 'private', bool $kill_all = TRUE, string $key = ''): void {

    $collection = trim($collection);
    if (empty($collection)) {
      throw new \InvalidArgumentException('Collection name cannot be empty.');
    }

    // Validate tempstore type.
    if (!in_array($type, ['private', 'shared'], TRUE)) {
      throw new InvalidTempstoreTypeException($type);
    }

    // Validate key parameter when not killing all.
    if (!$kill_all && empty($key)) {
      throw new \InvalidArgumentException('Key cannot be empty when not killing all items.');
    }

    if ($kill_all) {
      $this->killAllItems($collection, $type);
    }
    elseif ($type === 'private') {
      $this->killPrivateItem($collection, $key);
    }
    elseif ($type === 'shared') {
      $this->killSharedItem($collection, $key);
    }
  }

  /**
   * Kills all items in a temporary storage collection.
   *
   * @param string $collection
   *   The collection name.
   * @param string $type
   *   The storage type.
   *
   * @throws \Drupal\tsk\Exception\TempstoreDatabaseException
   *   When database operations fail.
   */
  protected function killAllItems(string $collection, string $type): void {
    try {
      $storage = $this->storageFactory->get("tempstore.$type.$collection");
      $storage->deleteAll();
      $this->loggerFactory->get('tsk')->info('Successfully deleted all temporary storage items in collection "@collection" of type "@type".', [
        '@collection' => $collection,
        '@type' => $type,
      ]);
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('tsk')->error('Database error while deleting all temporary storage items in collection "@collection" of type "@type": @message', [
        '@collection' => $collection,
        '@type' => $type,
        '@message' => $e->getMessage(),
      ]);
      throw new TempstoreDatabaseException("delete all items from collection '$collection'", $e->getMessage(), $e);
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('tsk')->error('Failed to delete all temporary storage items in collection "@collection" of type "@type": @message', [
        '@collection' => $collection,
        '@type' => $type,
        '@message' => $e->getMessage(),
      ]);
      throw $e;
    }
  }

  /**
   * Kills a specific private temporary storage item.
   *
   * @param string $collection
   *   The collection name.
   * @param string $key
   *   The item key.
   *
   * @throws \Drupal\tsk\Exception\TempstoreKeyNotFoundException
   *   When the key is not found.
   * @throws \Drupal\tsk\Exception\TempstoreDatabaseException
   *   When database operations fail.
   */
  protected function killPrivateItem(string $collection, string $key): void {
    try {
      // The delete method for private temp stores (privateTempStoreFactory)
      // does not work correctly. Therefore, the storage factory is used
      // instead.
      $storage = $this->storageFactory->get("tempstore.private.$collection");

      // Check if the key exists before attempting to delete.
      if (!$storage->has($key)) {
        throw new TempstoreKeyNotFoundException($key, $collection, 'private');
      }

      $storage->delete($key);
      $this->loggerFactory->get('tsk')->info('Successfully deleted private temporary storage item "@key" in collection "@collection".', [
        '@key' => $key,
        '@collection' => $collection,
      ]);
    }
    catch (TempstoreKeyNotFoundException $e) {
      $this->loggerFactory->get('tsk')->warning('Private temporary storage item "@key" was not found in collection "@collection".', [
        '@key' => $key,
        '@collection' => $collection,
      ]);
      throw $e;
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('tsk')->error('Database error while deleting private temporary storage item "@key" in collection "@collection": @message', [
        '@key' => $key,
        '@collection' => $collection,
        '@message' => $e->getMessage(),
      ]);
      throw new TempstoreDatabaseException("delete private item '$key' from collection '$collection'", $e->getMessage(), $e);
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('tsk')->error('Failed to delete private temporary storage item "@key" in collection "@collection": @message', [
        '@key' => $key,
        '@collection' => $collection,
        '@message' => $e->getMessage(),
      ]);
      throw $e;
    }
  }

  /**
   * Kills a specific shared temporary storage item.
   *
   * @param string $collection
   *   The collection name.
   * @param string $key
   *   The item key.
   *
   * @throws \Drupal\tsk\Exception\TempstoreKeyNotFoundException
   *   When the key is not found.
   * @throws \Drupal\tsk\Exception\TempstoreDatabaseException
   *   When database operations fail.
   */
  protected function killSharedItem(string $collection, string $key): void {
    try {
      $storage = $this->sharedTempStoreFactory->get($collection);

      // Check if the key exists before attempting to delete.
      if (!$storage->get($key)) {
        throw new TempstoreKeyNotFoundException($key, $collection, 'shared');
      }

      $storage->delete($key);
      $this->loggerFactory->get('tsk')->info('Successfully deleted shared temporary storage item "@key" in collection "@collection".', [
        '@key' => $key,
        '@collection' => $collection,
      ]);
    }
    catch (TempstoreKeyNotFoundException $e) {
      $this->loggerFactory->get('tsk')->warning('Shared temporary storage item "@key" was not found in collection "@collection".', [
        '@key' => $key,
        '@collection' => $collection,
      ]);
      throw $e;
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('tsk')->error('Database error while deleting shared temporary storage item "@key" in collection "@collection": @message', [
        '@key' => $key,
        '@collection' => $collection,
        '@message' => $e->getMessage(),
      ]);
      throw new TempstoreDatabaseException("delete shared item '$key' from collection '$collection'", $e->getMessage(), $e);
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('tsk')->error('Failed to delete shared temporary storage item "@key" in collection "@collection": @message', [
        '@key' => $key,
        '@collection' => $collection,
        '@message' => $e->getMessage(),
      ]);
      throw $e;
    }
  }

}
