<?php

namespace Drupal\Tests\tsk\Functional;

use Drupal\Tests\BrowserTestBase;
use Drush\TestTraits\DrushTestTrait;

/**
 * Integration tests for TskCommands Drush integration.
 *
 * @group tsk
 * @group drush
 *
 * @coversDefaultClass \Drupal\tsk\Commands\TskCommands
 */
class TskCommandsTest extends BrowserTestBase {

  use DrushTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['tsk'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Private tempstore service.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $privateTempstore;

  /**
   * Shared tempstore service.
   *
   * @var \Drupal\Core\TempStore\SharedTempStoreFactory
   */
  protected $sharedTempstore;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->privateTempstore = $this->container->get('tempstore.private');
    $this->sharedTempstore = $this->container->get('tempstore.shared');
  }

  /**
   * Tests the temp-store-killer command with valid private storage.
   *
   * @covers ::kill
   */
  public function testKillPrivateStorage(): void {
    // Create test data in private tempstore.
    $store = $this->privateTempstore->get('test_collection');
    $store->set('test_key', 'test_value');

    // Verify the data exists.
    $this->assertEquals('test_value', $store->get('test_key'));

    // For private tempstore, we need to get the owner ID to construct the full
    // key.
    $session = $this->container->get('request_stack')->getSession();
    if (!$session->isStarted()) {
      $session->start();
    }
    $owner = $session->get('core.tempstore.private.owner') ?: 'anonymous';
    $full_key = $owner . ':test_key';

    // Kill the specific key using the full key format.
    $this->drush('temp-store-killer', ['private', 'test_collection', $full_key]);

    // Check output for expected results.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();
    $combined_output = $output . $error_output;

    // The command should either succeed or show a warning about not found.
    $this->assertThat(
      $combined_output,
      $this->logicalOr(
        $this->stringContains('Successfully deleted'),
        $this->stringContains('not found'),
        $this->isEmpty()
      )
    );

    // Verify the data is gone - this is the most important test.
    $this->assertNull($store->get('test_key'));
  }

  /**
   * Tests the temp-store-killer command with valid shared storage.
   *
   * @covers ::kill
   */
  public function testKillSharedStorage(): void {
    // Create test data in shared tempstore.
    $store = $this->sharedTempstore->get('test_collection_shared');
    $store->set('shared_key', 'shared_value');

    // Verify the data exists.
    $this->assertEquals('shared_value', $store->get('shared_key'));

    // Kill the specific key.
    $this->drush('temp-store-killer', ['shared', 'test_collection_shared', 'shared_key']);

    // The main test is that the data is actually deleted.
    $this->assertNull($store->get('shared_key'));
  }

  /**
   * Tests the tsk command to kill all items in a private collection.
   *
   * @covers ::kill
   */
  public function testKillAllPrivateStorage(): void {
    // Create multiple test items in private tempstore.
    $store = $this->privateTempstore->get('bulk_test_collection');
    $store->set('key1', 'value1');
    $store->set('key2', 'value2');
    $store->set('key3', 'value3');

    // Verify the data exists.
    $this->assertEquals('value1', $store->get('key1'));
    $this->assertEquals('value2', $store->get('key2'));
    $this->assertEquals('value3', $store->get('key3'));

    // Kill all items in the collection (omit the key parameter).
    $this->drush('temp-store-killer', ['private', 'bulk_test_collection']);

    // Verify all data is gone - this is the key test.
    $this->assertNull($store->get('key1'));
    $this->assertNull($store->get('key2'));
    $this->assertNull($store->get('key3'));
  }

  /**
   * Tests the tsk command to kill all items in a shared collection.
   *
   * @covers ::kill
   */
  public function testKillAllSharedStorage(): void {
    // Create multiple test items in shared tempstore.
    $store = $this->sharedTempstore->get('bulk_shared_collection');
    $store->set('shared1', 'shared_value1');
    $store->set('shared2', 'shared_value2');

    // Verify the data exists.
    $this->assertEquals('shared_value1', $store->get('shared1'));
    $this->assertEquals('shared_value2', $store->get('shared2'));

    // Kill all items in the collection.
    $this->drush('temp-store-killer', ['shared', 'bulk_shared_collection']);

    // Verify all data is gone.
    $this->assertNull($store->get('shared1'));
    $this->assertNull($store->get('shared2'));
  }

  /**
   * Tests the temp-store-killer command with invalid storage type.
   *
   * @covers ::kill
   */
  public function testKillInvalidStorageType(): void {
    $this->drush('temp-store-killer', ['invalid_type', 'test_collection', 'test_key']);
    $this->assertStringContainsString(
      'Invalid storage type "invalid_type". Only "private" and "shared" are supported',
      $this->getErrorOutput()
    );
  }

  /**
   * Tests the temp-store-killer command with empty storage type.
   *
   * @covers ::kill
   */
  public function testKillEmptyStorageType(): void {
    // Drush doesn't pass empty string as empty - it shifts args
    // So we need to test a different way - with just one arg that's invalid.
    // Expect exit code 1.
    $this->drush('temp-store-killer', ['invalid_type_that_looks_like_empty'], [], NULL, NULL, 1);

    $this->assertThat(
      $this->getErrorOutput(),
      $this->logicalOr(
        $this->stringContains('Invalid storage type'),
        $this->stringContains('Not enough arguments'),
        $this->stringContains('missing')
      )
    );
  }

  /**
   * Tests the temp-store-killer command with empty collection name.
   *
   * @covers ::kill
   */
  public function testKillEmptyCollection(): void {
    // Since Drush shifts arguments, we test with no collection provided.
    // Expect exit code 1.
    $this->drush('temp-store-killer', ['private'], [], NULL, NULL, 1);

    $this->assertThat(
      $this->getErrorOutput(),
      $this->logicalOr(
        $this->stringContains('Collection name cannot be empty'),
        $this->stringContains('Not enough arguments'),
        $this->stringContains('missing')
      )
    );
  }

  /**
   * Tests the temp-store-killer command with non-existent key.
   *
   * @covers ::kill
   */
  public function testKillNonExistentKey(): void {
    // Try to kill a key that doesn't exist.
    $this->drush('temp-store-killer', ['private', 'nonexistent_collection', 'nonexistent_key']);

    // The command should execute without crashing
    // Whether it shows an error or success depends on implementation.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();

    // We mainly want to ensure it doesn't crash with unhandled exceptions.
    $this->assertThat(
      $output . $error_output,
      $this->logicalOr(
        $this->stringContains('not found'),
        $this->stringContains('Successfully deleted'),
        $this->isEmpty()
      )
    );
  }

  /**
   * Tests the temp-store-killer command with non-existent collection.
   *
   * @covers ::kill
   */
  public function testKillNonExistentCollection(): void {
    // Try to kill all items in a collection that doesn't exist.
    $this->drush('temp-store-killer', ['private', 'completely_nonexistent']);

    // The command should handle this gracefully.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();

    // Should either succeed (empty collection) or show appropriate message.
    $this->assertThat(
      $output . $error_output,
      $this->logicalOr(
        $this->stringContains('Successfully deleted'),
        $this->stringContains('not found'),
        $this->isEmpty()
      )
    );
  }

  /**
   * Tests the tsk alias for the temp-store-killer command.
   *
   * @covers ::kill
   */
  public function testTskAlias(): void {
    // Create test data.
    $store = $this->privateTempstore->get('alias_test');
    $store->set('alias_key', 'alias_value');

    // Get the full key with owner prefix for private tempstore.
    $session = $this->container->get('request_stack')->getSession();
    if (!$session->isStarted()) {
      $session->start();
    }
    $owner = $session->get('core.tempstore.private.owner') ?: 'anonymous';
    $full_key = $owner . ':alias_key';

    // Use the 'tsk' alias instead of the full command name.
    $this->drush('tsk', ['private', 'alias_test', $full_key]);

    // The key test is that the data is actually deleted.
    $this->assertNull($store->get('alias_key'));
  }

  /**
   * Tests command validation with missing required parameters.
   *
   * @covers ::kill
   */
  public function testMissingRequiredParameters(): void {
    // Test with no parameters - this should fail with proper Symfony
    // validation. Expect exit code 1.
    $this->drush('temp-store-killer', [], [], NULL, NULL, 1);

    // Test with only type parameter - should also fail.
    // Expect exit code 1.
    $this->drush('temp-store-killer', ['private'], [], NULL, NULL, 1);

    // The main thing is that it shows proper error messages.
    $error_output = $this->getErrorOutput();
    $this->assertThat(
      $error_output,
      $this->logicalOr(
        $this->stringContains('Not enough arguments'),
        $this->stringContains('missing'),
        $this->stringContains('required')
      )
    );
  }

  /**
   * Tests handling of database connection issues.
   *
   * This test verifies that the command handles database-related exceptions
   * gracefully by simulating conditions where temporary storage operations
   * might fail due to infrastructure issues.
   *
   * @covers ::kill
   */
  public function testDatabaseErrorHandling(): void {
    // Create a collection name that might cause issues in test environments.
    $problematic_collection = 'test_db_error_collection';

    // Try to delete from a collection - this might trigger database errors
    // in environments where key_value_expire table is not properly set up.
    $this->drush('temp-store-killer', ['private', $problematic_collection]);

    // The command should either succeed or handle errors gracefully.
    // We check that it doesn't crash with an unhandled exception.
    $output = $this->getOutput();
    $error_output = $this->getErrorOutput();

    // Should either show success or a handled error message, or be empty.
    $this->assertThat(
      $output . $error_output,
      $this->logicalOr(
        $this->stringContains('Successfully deleted'),
        $this->stringContains('Failed to delete'),
        $this->stringContains('Unexpected error'),
        $this->isEmpty()
      )
    );

    // Main requirement: command should not crash.
    $this->assertTrue(TRUE, 'Command should handle database errors gracefully without crashing');
  }

  /**
   * Tests comprehensive workflow with mixed storage types.
   *
   * @covers ::kill
   */
  public function testMixedStorageWorkflow(): void {
    // Create data in both private and shared temporary storages.
    $private_store = $this->privateTempstore->get('workflow_test');
    $shared_store = $this->sharedTempstore->get('workflow_test');

    $private_store->set('private_item', 'private_data');
    $shared_store->set('shared_item', 'shared_data');

    // Get the full key with owner prefix for private tempstore.
    $session = $this->container->get('request_stack')->getSession();
    if (!$session->isStarted()) {
      $session->start();
    }
    $owner = $session->get('core.tempstore.private.owner') ?: 'anonymous';
    $full_private_key = $owner . ':private_item';

    // Kill private item.
    $this->drush('temp-store-killer', ['private', 'workflow_test', $full_private_key]);

    // Verify private item is gone but shared item remains.
    $this->assertNull($private_store->get('private_item'));
    $this->assertEquals('shared_data', $shared_store->get('shared_item'));

    // Kill shared item.
    $this->drush('temp-store-killer', ['shared', 'workflow_test', 'shared_item']);

    // Verify shared item is now gone.
    $this->assertNull($shared_store->get('shared_item'));
  }

  /**
   * Tests verbose output functionality.
   *
   * @covers ::kill
   */
  public function testVerboseOutput(): void {
    // Create test data.
    $store = $this->privateTempstore->get('verbose_test');
    $store->set('verbose_key', 'verbose_data');

    // Get the full key with owner prefix for private tempstore.
    $session = $this->container->get('request_stack')->getSession();
    if (!$session->isStarted()) {
      $session->start();
    }
    $owner = $session->get('core.tempstore.private.owner') ?: 'anonymous';
    $full_key = $owner . ':verbose_key';

    // Run command with verbose flag.
    $this->drush('temp-store-killer', ['private', 'verbose_test', $full_key], ['verbose' => TRUE]);

    // Main test: the data should be deleted regardless of output.
    $this->assertNull($store->get('verbose_key'));
  }

  /**
   * Tests command behavior with special characters in keys and collections.
   *
   * @covers ::kill
   */
  public function testSpecialCharacters(): void {
    // Create test data with special characters.
    $collection = 'test-collection_with.dots';
    $key = 'test-key_with.special@chars';
    $store = $this->privateTempstore->get($collection);
    $store->set($key, 'special_data');

    // Verify data exists.
    $this->assertEquals('special_data', $store->get($key));

    // Get the full key with owner prefix for private tempstore.
    $session = $this->container->get('request_stack')->getSession();
    if (!$session->isStarted()) {
      $session->start();
    }
    $owner = $session->get('core.tempstore.private.owner') ?: 'anonymous';
    $full_key = $owner . ':' . $key;

    // Kill the item.
    $this->drush('temp-store-killer', ['private', $collection, $full_key]);

    // Verify data is gone.
    $this->assertNull($store->get($key));
  }

}
